@@ -11,7 +11,6 @@
 #include <errno.h>
 
 #include "checkers.h"
-#include "memory.h"
 #include "util.h"
 #include "debug.h"
 #include "parser.h"
@@ -238,6 +237,18 @@ const char *get_mpe_wwid(const struct _vector *mptable, const char *alias)
 	return NULL;
 }
 
+static void
+free_pctable (vector pctable)
+{
+	int i;
+	struct pcentry *pce;
+
+	vector_foreach_slot(pctable, pce, i)
+		free(pce);
+
+	vector_free(pctable);
+}
+
 void
 free_hwe (struct hwentry * hwe)
 {
@@ -245,45 +256,45 @@ free_hwe (struct hwentry * hwe)
 		return;
 
 	if (hwe->vendor)
-		FREE(hwe->vendor);
+		free(hwe->vendor);
 
 	if (hwe->product)
-		FREE(hwe->product);
+		free(hwe->product);
 
 	if (hwe->revision)
-		FREE(hwe->revision);
-
-	if (hwe->getuid)
-		FREE(hwe->getuid);
+		free(hwe->revision);
 
 	if (hwe->uid_attribute)
-		FREE(hwe->uid_attribute);
+		free(hwe->uid_attribute);
 
 	if (hwe->features)
-		FREE(hwe->features);
+		free(hwe->features);
 
 	if (hwe->hwhandler)
-		FREE(hwe->hwhandler);
+		free(hwe->hwhandler);
 
 	if (hwe->selector)
-		FREE(hwe->selector);
+		free(hwe->selector);
 
 	if (hwe->checker_name)
-		FREE(hwe->checker_name);
+		free(hwe->checker_name);
 
 	if (hwe->prio_name)
-		FREE(hwe->prio_name);
+		free(hwe->prio_name);
 
 	if (hwe->prio_args)
-		FREE(hwe->prio_args);
+		free(hwe->prio_args);
 
 	if (hwe->alias_prefix)
-		FREE(hwe->alias_prefix);
+		free(hwe->alias_prefix);
 
 	if (hwe->bl_product)
-		FREE(hwe->bl_product);
+		free(hwe->bl_product);
 
-	FREE(hwe);
+	if (hwe->pctable)
+		free_pctable(hwe->pctable);
+
+	free(hwe);
 }
 
 void
@@ -308,27 +319,24 @@ free_mpe (struct mpentry * mpe)
 		return;
 
 	if (mpe->wwid)
-		FREE(mpe->wwid);
+		free(mpe->wwid);
 
 	if (mpe->selector)
-		FREE(mpe->selector);
-
-	if (mpe->getuid)
-		FREE(mpe->getuid);
+		free(mpe->selector);
 
 	if (mpe->uid_attribute)
-		FREE(mpe->uid_attribute);
+		free(mpe->uid_attribute);
 
 	if (mpe->alias)
-		FREE(mpe->alias);
+		free(mpe->alias);
 
 	if (mpe->prio_name)
-		FREE(mpe->prio_name);
+		free(mpe->prio_name);
 
 	if (mpe->prio_args)
-		FREE(mpe->prio_args);
+		free(mpe->prio_args);
 
-	FREE(mpe);
+	free(mpe);
 }
 
 void
@@ -350,7 +358,7 @@ struct mpentry *
 alloc_mpe (void)
 {
 	struct mpentry * mpe = (struct mpentry *)
-				MALLOC(sizeof(struct mpentry));
+				calloc(1, sizeof(struct mpentry));
 
 	return mpe;
 }
@@ -359,11 +367,20 @@ struct hwentry *
 alloc_hwe (void)
 {
 	struct hwentry * hwe = (struct hwentry *)
-				MALLOC(sizeof(struct hwentry));
+				calloc(1, sizeof(struct hwentry));
 
 	return hwe;
 }
 
+struct pcentry *
+alloc_pce (void)
+{
+	struct pcentry *pce = (struct pcentry *)
+				calloc(1, sizeof(struct pcentry));
+	pce->type = PCE_INVALID;
+	return pce;
+}
+
 static char *
 set_param_str(const char * str)
 {
@@ -378,7 +395,7 @@ set_param_str(const char * str)
 	if (!len)
 		return NULL;
 
-	dst = (char *)MALLOC(len + 1);
+	dst = (char *)calloc(1, len + 1);
 
 	if (!dst)
 		return NULL;
@@ -388,24 +405,30 @@ set_param_str(const char * str)
 }
 
 #define merge_str(s) \
-	if (!dst->s && src->s) { \
-		if (!(dst->s = set_param_str(src->s))) \
-			return 1; \
+	if (!dst->s && src->s && strlen(src->s)) { \
+		dst->s = src->s; \
+		src->s = NULL; \
 	}
 
 #define merge_num(s) \
 	if (!dst->s && src->s) \
 		dst->s = src->s
 
+static void
+merge_pce(struct pcentry *dst, struct pcentry *src)
+{
+	merge_num(fast_io_fail);
+	merge_num(dev_loss);
+	merge_num(eh_deadline);
+}
 
-static int
+static void
 merge_hwe (struct hwentry * dst, struct hwentry * src)
 {
 	char id[SCSI_VENDOR_SIZE+PATH_PRODUCT_SIZE];
 	merge_str(vendor);
 	merge_str(product);
 	merge_str(revision);
-	merge_str(getuid);
 	merge_str(uid_attribute);
 	merge_str(features);
 	merge_str(hwhandler);
@@ -450,18 +473,13 @@ merge_hwe (struct hwentry * dst, struct hwentry * src)
 	reconcile_features_with_options(id, &dst->features,
 					&dst->no_path_retry,
 					&dst->retain_hwhandler);
-	return 0;
 }
 
-static int
+static void
 merge_mpe(struct mpentry *dst, struct mpentry *src)
 {
-	if (!dst || !src)
-		return 1;
-
 	merge_str(alias);
 	merge_str(uid_attribute);
-	merge_str(getuid);
 	merge_str(selector);
 	merge_str(features);
 	merge_str(prio_name);
@@ -500,8 +518,6 @@ merge_mpe(struct mpentry *dst, struct mpentry *src)
 	merge_num(uid);
 	merge_num(gid);
 	merge_num(mode);
-
-	return 0;
 }
 
 void merge_mptable(vector mptable)
@@ -555,9 +571,6 @@ store_hwe (vector hwtable, struct hwentry * dhwe)
 	if (dhwe->uid_attribute && !(hwe->uid_attribute = set_param_str(dhwe->uid_attribute)))
 		goto out;
 
-	if (dhwe->getuid && !(hwe->getuid = set_param_str(dhwe->getuid)))
-		goto out;
-
 	if (dhwe->features && !(hwe->features = set_param_str(dhwe->features)))
 		goto out;
 
@@ -609,6 +622,51 @@ out:
 	return 1;
 }
 
+static void
+validate_pctable(struct hwentry *ovr, int idx, const char *table_desc)
+{
+	struct pcentry *pce;
+
+	if (!ovr || !ovr->pctable)
+		return;
+
+	vector_foreach_slot_after(ovr->pctable, pce, idx) {
+		if (pce->type == PCE_INVALID) {
+			condlog(0, "protocol section in %s missing type",
+				table_desc);
+			vector_del_slot(ovr->pctable, idx--);
+			free(pce);
+		}
+	}
+
+	if (VECTOR_SIZE(ovr->pctable) == 0) {
+		vector_free(ovr->pctable);
+		ovr->pctable = NULL;
+	}
+}
+
+static void
+merge_pctable(struct hwentry *ovr)
+{
+	struct pcentry *pce1, *pce2;
+	int i, j;
+
+	if (!ovr || !ovr->pctable)
+		return;
+
+	vector_foreach_slot(ovr->pctable, pce1, i) {
+		j = i + 1;
+		vector_foreach_slot_after(ovr->pctable, pce2, j) {
+			if (pce1->type != pce2->type)
+				continue;
+			merge_pce(pce2,pce1);
+			vector_del_slot(ovr->pctable, i--);
+			free(pce1);
+			break;
+		}
+	}
+}
+
 static void
 factorize_hwtable (vector hw, int n, const char *table_desc)
 {
@@ -652,61 +710,58 @@ restart:
 
 static struct config *alloc_config (void)
 {
-	return (struct config *)MALLOC(sizeof(struct config));
+	return (struct config *)calloc(1, sizeof(struct config));
 }
 
 static void _uninit_config(struct config *conf)
 {
+	void *ptr;
+	int i;
+
 	if (!conf)
 		conf = &__internal_config;
 
-	if (conf->multipath_dir)
-		FREE(conf->multipath_dir);
-
 	if (conf->selector)
-		FREE(conf->selector);
+		free(conf->selector);
 
 	if (conf->uid_attribute)
-		FREE(conf->uid_attribute);
+		free(conf->uid_attribute);
 
+	vector_foreach_slot(&conf->uid_attrs, ptr, i)
+		free(ptr);
 	vector_reset(&conf->uid_attrs);
 
-	if (conf->getuid)
-		FREE(conf->getuid);
-
 	if (conf->features)
-		FREE(conf->features);
+		free(conf->features);
 
 	if (conf->hwhandler)
-		FREE(conf->hwhandler);
+		free(conf->hwhandler);
 
 	if (conf->bindings_file)
-		FREE(conf->bindings_file);
+		free(conf->bindings_file);
 
 	if (conf->wwids_file)
-		FREE(conf->wwids_file);
+		free(conf->wwids_file);
 
 	if (conf->prkeys_file)
-		FREE(conf->prkeys_file);
+		free(conf->prkeys_file);
 
 	if (conf->prio_name)
-		FREE(conf->prio_name);
+		free(conf->prio_name);
 
 	if (conf->alias_prefix)
-		FREE(conf->alias_prefix);
+		free(conf->alias_prefix);
 	if (conf->partition_delim)
-		FREE(conf->partition_delim);
+		free(conf->partition_delim);
 
 	if (conf->prio_args)
-		FREE(conf->prio_args);
+		free(conf->prio_args);
 
 	if (conf->checker_name)
-		FREE(conf->checker_name);
+		free(conf->checker_name);
 
-	if (conf->config_dir)
-		FREE(conf->config_dir);
 	if (conf->enable_foreign)
-		FREE(conf->enable_foreign);
+		free(conf->enable_foreign);
 
 	free_blacklist(conf->blist_devnode);
 	free_blacklist(conf->blist_wwid);
@@ -757,6 +812,7 @@ process_config_dir(struct config *conf, char *dir)
 	int i, n;
 	char path[LINE_MAX];
 	int old_hwtable_size;
+	int old_pctable_size = 0;
 
 	if (dir[0] != '/') {
 		condlog(1, "config_dir '%s' must be a fully qualified path",
@@ -783,11 +839,15 @@ process_config_dir(struct config *conf, char *dir)
 			continue;
 
 		old_hwtable_size = VECTOR_SIZE(conf->hwtable);
+		old_pctable_size = conf->overrides ?
+				   VECTOR_SIZE(conf->overrides->pctable) : 0;
 		snprintf(path, LINE_MAX, "%s/%s", dir, namelist[i]->d_name);
 		path[LINE_MAX-1] = '\0';
 		process_file(conf, path);
 		factorize_hwtable(conf->hwtable, old_hwtable_size,
 				  namelist[i]->d_name);
+		validate_pctable(conf->overrides, old_pctable_size,
+				 namelist[i]->d_name);
 	}
 	pthread_cleanup_pop(1);
 }
@@ -850,7 +910,6 @@ int _init_config (const char *file, struct config *conf)
 	conf->bindings_file = set_default(DEFAULT_BINDINGS_FILE);
 	conf->wwids_file = set_default(DEFAULT_WWIDS_FILE);
 	conf->prkeys_file = set_default(DEFAULT_PRKEYS_FILE);
-	conf->multipath_dir = set_default(DEFAULT_MULTIPATHDIR);
 	conf->attribute_flags = 0;
 	conf->reassign_maps = DEFAULT_REASSIGN_MAPS;
 	conf->checkint = CHECKINT_UNDEF;
@@ -895,13 +954,11 @@ int _init_config (const char *file, struct config *conf)
 			goto out;
 		}
 		factorize_hwtable(conf->hwtable, builtin_hwtable_size, file);
+		validate_pctable(conf->overrides, 0, file);
 	}
 
 	conf->processed_main_config = 1;
-	if (conf->config_dir == NULL)
-		conf->config_dir = set_default(DEFAULT_CONFIG_DIR);
-	if (conf->config_dir && conf->config_dir[0] != '\0')
-		process_config_dir(conf, conf->config_dir);
+	process_config_dir(conf, CONFIG_DIR);
 
 	/*
 	 * fill the voids left in the config file
@@ -995,6 +1052,7 @@ int _init_config (const char *file, struct config *conf)
 			goto out;
 	}
 
+	merge_pctable(conf->overrides);
 	merge_mptable(conf->mptable);
 	merge_blacklist(conf->blist_devnode);
 	merge_blacklist(conf->blist_property);
@@ -1008,8 +1066,7 @@ int _init_config (const char *file, struct config *conf)
 	if (conf->bindings_file == NULL)
 		conf->bindings_file = set_default(DEFAULT_BINDINGS_FILE);
 
-	if (!conf->multipath_dir || !conf->bindings_file ||
-	    !conf->wwids_file || !conf->prkeys_file)
+	if (!conf->bindings_file || !conf->wwids_file || !conf->prkeys_file)
 		goto out;
 
 	libmp_verbosity = conf->verbosity;
@@ -1019,10 +1076,10 @@ out:
 	return 1;
 }
 
-char *get_uid_attribute_by_attrs(struct config *conf,
-				 const char *path_dev)
+const char *get_uid_attribute_by_attrs(const struct config *conf,
+				       const char *path_dev)
 {
-	vector uid_attrs = &conf->uid_attrs;
+	const struct _vector *uid_attrs = &conf->uid_attrs;
 	int j;
 	char *att, *col;
 
